package PureMessage::Policy::Router;

use strict;
use warnings;

use base 'PureMessage::Policy::Plugin';

use PureMessage::Config qw(%config);
use PureMessage::Queue qw(queue_mail);

use Net::Domain qw(hostfqdn);

our @EXPORT_OK = qw(pmx_route);
our $VERSION   = '5.500';

my %commands = (
    pmx_route => \&pmx_route,
);

sub new {
    my $class = shift;
    my %opts = @_;
    return bless \%opts, $class;
}

sub commands {
    my $self = shift;
    return %commands;
}

sub pmx_route {
    my ($msg, $args) = @_;
    my $servers = $args->param(0);
    
    $msg->assert_event('eom');
    $msg->minfo_value('Reason', 'Route');

    $msg->minfo_value('Bounce-On-Failure', 'Yes');

    if ($args->tag('copy')) {
        $msg->minfo_value('Route-Copy', 'Yes');
        $msg->delivery_action('keep');
    }

    my $nobounce = '';
    if($args->tag('nobounce')) {
        $nobounce = ($config{'nobounce-addr'} =~ m/@/) ? $config{'nobounce-addr'} : "$config{'nobounce-addr'}\@".hostfqdn();
    }

    for my $server (@$servers) {

        $msg->minfo_value('Destination-Server', "$server");

        # if the server is an IP address, we need to wrap it in brackets to be a
        # valid email address.
        if ($server =~ m/(?:\d{1,3}\.){3}\d{1,3}/) {
            $server = "[$server]";
        }

        my @to = $msg->envelope_recipients();
        for (@to) {
            s/@/%/;
            $_ .= "\@$server";
        }

        if ($args->tag('copy')) {
            $msg->log('DEBUG', "pmx_route($server): copying to queue");
        }
        else {  # route
            $msg->log('DEBUG', "pmx_route($server): moving to queue");
            $msg->delivery_action('route', "$server");
        }

        # Put routed message into queue 
        if($args->tag('nobounce')) {
            queue_mail(tempfile => $msg->implode_to_tempfile(), to=>\@to, from=>$nobounce, minfo => $msg->minfo());
        }
        else {
            queue_mail(tempfile => $msg->implode_to_tempfile(), to=>\@to, minfo => $msg->minfo());
        }
    }
    my $mark = $args->tag('copy') ? 'copy' : 'route';
    $msg->set_attr("mark_$mark", join(',', @$servers));
    $msg->stop() unless $args->tag('copy');
}

1;

